@@ -1,9 +1,10 @@
 from pytest import raises
 import sqlalchemy as sa
+from sqlalchemy_utils import EmailType
 from wtforms.validators import (
-    Email, InputRequired, DataRequired
+    Email, InputRequired, DataRequired, Optional
 )
-from wtforms_alchemy import Unique, ModelForm
+from wtforms_alchemy import Unique, ModelForm, ClassMap
 from tests import ModelFormTestCase
 
 
@@ -36,7 +37,7 @@ class TestAutoAssignedValidators(ModelFormTestCase):
         self.assert_has_validator('test_column', DataRequired)
         self.assert_has_validator('test_column', InputRequired)
 
-    def test_string_not_nullable_validator_fallback(self):
+    def test_type_level_not_nullable_validators(self):
         class ModelTest(self.base):
             __tablename__ = 'model_test'
             id = sa.Column(sa.Integer, primary_key=True)
@@ -47,13 +48,32 @@ class TestAutoAssignedValidators(ModelFormTestCase):
         class ModelTestForm(ModelForm):
             class Meta:
                 model = ModelTest
-                not_null_str_validator = None
+                not_null_validator_type_map = ClassMap()
                 not_null_validator = validator
 
         form = ModelTestForm()
         assert validator in form.test_column.validators
 
-    def test_configure_not_null_validator(self):
+    def test_not_nullable_validator_with_type_decorator(self):
+        class ModelTest(self.base):
+            __tablename__ = 'model_test'
+            id = sa.Column(sa.Integer, primary_key=True)
+            test_column = sa.Column(EmailType, nullable=False)
+
+        validator = DataRequired()
+
+        class ModelTestForm(ModelForm):
+            class Meta:
+                model = ModelTest
+                not_null_validator_type_map = ClassMap(
+                    [(sa.String, validator)]
+                )
+                not_null_validator = []
+
+        form = ModelTestForm()
+        assert validator in form.test_column.validators
+
+    def test_not_null_validator_as_empty_list(self):
         class ModelTest(self.base):
             __tablename__ = 'model_test'
             id = sa.Column(sa.Integer, primary_key=True)
@@ -62,12 +82,28 @@ class TestAutoAssignedValidators(ModelFormTestCase):
         class ModelTestForm(ModelForm):
             class Meta:
                 model = ModelTest
-                not_null_str_validator = None
-                not_null_validator = None
+                not_null_validator_type_map = ClassMap()
+                not_null_validator = []
 
         form = ModelTestForm()
         assert form.test_column.validators == []
 
+    def test_not_null_validator_as_none(self):
+        class ModelTest(self.base):
+            __tablename__ = 'model_test'
+            id = sa.Column(sa.Integer, primary_key=True)
+            test_column = sa.Column(sa.Boolean, nullable=False)
+
+        class ModelTestForm(ModelForm):
+            class Meta:
+                model = ModelTest
+                not_null_validator_type_map = ClassMap()
+                not_null_validator = None
+
+        form = ModelTestForm()
+        assert len(form.test_column.validators) == 1
+        assert isinstance(form.test_column.validators[0], Optional)
+
     def test_not_nullable_booleans_are_required(self):
         self.init(sa.Boolean, nullable=False)
         self.assert_has_validator('test_column', InputRequired)
