@@ -1,5 +1,6 @@
 #import pytz
 import six
+import sqlalchemy as sa
 from wtforms import Form
 from wtforms.validators import InputRequired, DataRequired
 from wtforms.form import FormMeta
@@ -17,11 +18,13 @@ from .utils import (
     is_scalar,
     null_or_int,
     null_or_unicode,
+    ClassMap
 )
 from .exc import (
     AttributeTypeException,
     InvalidAttributeException,
-    UnknownTypeException
+    UnknownTypeException,
+    UnknownConfigurationOption
 )
 from .fields import ModelFieldList, ModelFormField
 from .generator import FormGenerator
@@ -47,7 +50,7 @@ __all__ = (
 )
 
 
-__version__ = '0.9.3'
+__version__ = '0.10.0'
 
 
 class ModelFormMeta(FormMeta):
@@ -95,65 +98,70 @@ def model_form_factory(base=Form, meta=ModelFormMeta, **defaults):
             #:
             #: By default this is set to False, meaning unknown types throw
             #: exceptions when encountered.
-            skip_unknown_types = defaults.get('skip_unknown_types', False)
-
-            #: Whether or not to assign non-nullable fields as required
-            assign_required = defaults.get('assign_required', True)
+            skip_unknown_types = defaults.pop('skip_unknown_types', False)
 
             #: Whether or not to assign all fields as optional, useful when
             #: creating update forms for patch requests
-            all_fields_optional = defaults.get('all_fields_optional', False)
+            all_fields_optional = defaults.pop('all_fields_optional', False)
 
-            validators = {}
+            validators = defaults.pop('validators', {})
 
             #: A dict with keys as field names and values as field arguments.
-            field_args = {}
+            field_args = defaults.pop('field_args', {})
 
             #: A dict with keys as field names and values as widget options.
-            widget_options = {}
+            widget_options = defaults.pop('widget_options', {})
 
             #: Whether or not to include only indexed fields.
-            only_indexed_fields = defaults.get('only_indexed_fields', False)
+            only_indexed_fields = defaults.pop('only_indexed_fields', False)
 
             #: Whether or not to include primary keys.
-            include_primary_keys = defaults.get('include_primary_keys', False)
+            include_primary_keys = defaults.pop('include_primary_keys', False)
 
             #: Whether or not to include foreign keys. By default this is False
             #: indicating that foreign keys are not included in the generated
             #: form.
-            include_foreign_keys = defaults.get('include_foreign_keys', False)
+            include_foreign_keys = defaults.pop('include_foreign_keys', False)
 
             #: Whether or not to strip string fields
-            strip_string_fields = defaults.get('strip_string_fields', False)
+            strip_string_fields = defaults.pop('strip_string_fields', False)
 
             #: Whether or not to include datetime columns that have a default
             #: value. A good example is created_at column which has a default
             #: value of datetime.utcnow.
-            include_datetimes_with_default = defaults.get(
+            include_datetimes_with_default = defaults.pop(
                 'include_datetimes_with_default', False
             )
 
             #: The default validator to be used for not nullable columns. Set
             #: this to `None` if you wish to disable it.
-            not_null_validator = InputRequired()
+            not_null_validator = defaults.pop(
+                'not_null_validator',
+                InputRequired()
+            )
 
-            #: The default validator to be used for not nullable string
-            #: columns. If this is set to `None` the configuration option
-            #: `not_null_validator` will be used for string columns also.
-            not_null_str_validator = [InputRequired(), DataRequired()]
+            #: A dictionary that overrides not null validation on type level.
+            #: Keys should be valid SQLAlchemy types and values should be valid
+            #: WTForms validators.
+            not_null_validator_type_map = defaults.pop(
+                'not_null_validator_type_map',
+                ClassMap(
+                    [(sa.String, [InputRequired(), DataRequired()])]
+                )
+            )
 
             #: Which form generator to use. Only override this if you have a
             #: valid form generator which you want to use instead of the
             #: default one.
-            form_generator = defaults.get(
+            form_generator = defaults.pop(
                 'form_generator', FormGenerator
             )
 
             #: Default date format
-            date_format = defaults.get('date_format', '%Y-%m-%d')
+            date_format = defaults.pop('date_format', '%Y-%m-%d')
 
             #: Default datetime format
-            datetime_format = defaults.get(
+            datetime_format = defaults.pop(
                 'datetime_format', '%Y-%m-%d %H:%M:%S'
             )
 
@@ -163,16 +171,16 @@ def model_form_factory(base=Form, meta=ModelFormMeta, **defaults):
             #:
             #: Using this configuration option one can easily configure the
             #: type conversion in class level.
-            type_map = {}
+            type_map = defaults.pop('type_map', ClassMap())
 
             #: Additional fields to include in the generated form.
-            include = []
+            include = defaults.pop('include', [])
 
             #: List of fields to exclude from the generated form.
-            exclude = []
+            exclude = defaults.pop('exclude', [])
 
             #: List of fields to only include in the generated form.
-            only = []
+            only = defaults.pop('only', [])
 
         def __init__(self, *args, **kwargs):
             """Sets object as form attribute."""
@@ -180,6 +188,11 @@ def model_form_factory(base=Form, meta=ModelFormMeta, **defaults):
             self._obj = kwargs.get('obj', None)
             super(ModelForm, self).__init__(*args, **kwargs)
 
+    if defaults:
+        raise UnknownConfigurationOption(
+            list(defaults.keys())[0]
+        )
+
     return ModelForm
 
 
